/* +++Date last modified: 05-Jul-1997 */

/*
+----------------------------------------------------+
|               Thunderbird Software                 |
+----------------------------------------------------+
| Filespec  :  Serial.c                              |
| Date      :  October 24, 1991                      |
| Time      :  15:03                                 |
| Revision  :  1.1                                   |
|     Update: August 29, 1994                        |
+----------------------------------------------------+
| Programmer:  Scott Andrews                         |
| Address   :  5358 Summit RD SW                     |
| City/State:  Pataskala, Ohio                       |
| Zip       :  43062                                 |
+----------------------------------------------------+
| Released to the Public Domain                      |
+----------------------------------------------------+
| Modified to match ports of Portfolio               |
| Michael Bernstein                                  |
+----------------------------------------------------+
*/

/*
+----------------------------------------------------------+
|  Call open_serial to install the interrupt handler       |
|  You must call close_serial before exiting your program  |
|  or a machine crash will occur!                          |
+----------------------------------------------------------+
*/
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include "queue.h"
#include "serial.h"

#define SerInBufSize  4096            /* Size of input buffer          */
#define SerOutBufSize 512             /* Size of output buffer         */
#define ComBase 0x8070
#define IrqNum 4

/* 8250 registers */
#define  REC             0             /* Uart receive reg.             */
#define  XMIT            0             /* Uart transmit reg.            */
#define  INT_EN          1             /* Uart int. enable reg.         */
#define  INT_ID          2             /* Uart int. ident. reg.         */
#define  LINE_CNTRL      3             /* Uart line control reg.        */
#define  MODEM_CNTRL     4             /* Uart modem control reg.       */
#define  LINE_STATUS     5             /* Uart line status reg.         */
#define  MODEM_STATUS    6             /* Uart modem status reg.        */
#define  BAUD_LSB        0             /* Uart baud divisor reg.        */
#define  BAUD_MSB        1             /* Uart baud divisor reg.        */

/* Interrupt enable register  */
#define  RX_INT          0x01          /* Receive interrupt mask        */
#define  TBE_INT         0x02          /* Transmit buffer empty mask    */
#define  ERR_INT         0x04          /* Error interrupt mask          */
#define  RS_INT          0x08          /* Line interrupt mask           */

/* Interrupt id register      */
#define  OUT2            0x08          /* Out 2 line                    */
#define  DTR             0x01          /* DTR high                      */
#define  RTS             0x02          /* RTS high                      */
#define  CTS             0x10
#define  DSR             0x20
#define  XMTRDY          0x20
#define  TXR             0       /*  Transmit register (WRITE)          */

static QUEUE *Serial_In_Buffer;
static QUEUE *Serial_Out_Buffer;

OLD_COMM_PARAMS old_comm_params;
COMM_STATUS     comm_status;

static void (interrupt far *oldvector_serial)();  /* save addr for intr handler */

static void interrupt far serial( void );
extern void EnableInt(void);
extern void Disable(void);

int OpenComPort( void )        /* install int. handler */
{
   unsigned status;
   int retval = -1;
    
   /* allocate input and output queues */
   Serial_In_Buffer = QueueAlloc( SerInBufSize );
   if ((QUEUE *)NULL == Serial_In_Buffer)
      return retval;
   else
   {
      Serial_Out_Buffer = QueueAlloc( SerOutBufSize );
      if ((QUEUE *)NULL == Serial_Out_Buffer)
      {
         QueueFree(Serial_In_Buffer );
         return retval;
      }
      else
      {
         retval = 0;
         old_comm_params.int_enable = inp( ComBase + INT_EN );
         outp( ComBase + INT_EN, 0 ); /* turn off comm interrupts */
         /* save old comm parameters */
         old_comm_params.line = inp( ComBase + LINE_CNTRL );
         old_comm_params.modem = inp( ComBase + MODEM_CNTRL );
         status = inp( ComBase + LINE_CNTRL );
         outp( ComBase + LINE_CNTRL, (unsigned char)status | 0x80 );
         old_comm_params.baud_lsb = inp( ComBase + BAUD_LSB );
         old_comm_params.baud_msb = inp( ComBase + BAUD_MSB );
         status = inp( ComBase + LINE_CNTRL );
         outp( ComBase + LINE_CNTRL, (unsigned char)status | 0x7F );
         status = OUT2 | DTR;              /* DTR/OUT2 must be set! */
         outp( ComBase + MODEM_CNTRL, (unsigned char)status );
         /* get serial port address/vector */
         oldvector_serial = (void(interrupt far *)(void))getvect(IrqNum + 8);
         /* set our interrupt handler */
         setvect( IrqNum + 8, serial );
         /* ok enable comm ints */
         EnableInt();
         return retval;
      }
   }
}

void CloseComPort( void )
{
   int status;

   /* restore UART to previous state */
   outp( ComBase + INT_EN, (unsigned char)0 );
   outp( ComBase + MODEM_CNTRL, (unsigned char)old_comm_params.modem );
   status = inp( ComBase + LINE_CNTRL );
   outp( ComBase + LINE_CNTRL, (unsigned char)status | 0x80 );
   outp( ComBase + BAUD_LSB, (unsigned char)old_comm_params.baud_lsb );
   outp( ComBase + BAUD_MSB, (unsigned char)old_comm_params.baud_msb );
   outp( ComBase + LINE_CNTRL, (unsigned char)old_comm_params.line );
   DisableInt();
   /* restore old interrupt handler */
   setvect( IrqNum + 8, oldvector_serial );
   /* free input and output queues */
   QueueFree( Serial_In_Buffer );
   QueueFree( Serial_Out_Buffer );
}

void InitComPort( char Baud[], char Databits, char Parity, char Stopbits )
{
   int status;
   long baudrate;
   unsigned divisor;

   /* set baud rate */
   status = inp( ComBase + LINE_CNTRL );
   outp( ComBase + LINE_CNTRL, (unsigned char)status | 0x80 );
   baudrate = atol( Baud );
   if (baudrate == 0)
      baudrate = 2400L;
   divisor = (unsigned)(115200L / baudrate);
   outp( ComBase + BAUD_LSB, (unsigned char)( divisor & 0x00FF) );
   outp( ComBase + BAUD_MSB, (unsigned char)( ( divisor >> 8) & 0x00FF) );
   status = 0x00;
   /* set parity */
   switch (Parity)                    /* set parity value     */
   {
      case 'O':                           /* odd parity           */
      case 'o':
         status = 0x08; 
         break;
      case 'E':                           /* even parity          */
      case 'e':
         status = 0x18; 
         break;
      case 'S':                           /* stick parity         */
      case 's':
         status = 0x28; 
         break;
      case 'N':                           /* no parity            */
      case 'n':
      default:   
         status = 0x00;
   }
   /* set number data bits */
   switch (Databits)
   {
      case '5':
         break;
      case '6':
         status = status | 0x01;
         break;
      case '7':
         status = status | 0x02;
         break;
      case '8':
      default :
         status = status | 0x03;
   }
   /* set number stop bits */
   switch (Stopbits)
   {
      case '2':
         status = status | 0x04;
         break;
      case '1':
      default :
         ;
   }
   outp( ComBase + LINE_CNTRL, (unsigned char)status );
   status = OUT2 | DTR;        /* DTR/OUT2 must be set! */
   outp( ComBase + MODEM_CNTRL, (unsigned char)status );
   /* enable serial interrupts */
   outp( ComBase + INT_EN, RX_INT | ERR_INT | RS_INT );
}

static void DropDtr( void )
{
   int status;

   status = inp( ComBase + MODEM_CNTRL );
   status &= 0xFE;                 /* turn off DTR bit */
   outp( ComBase + MODEM_CNTRL, (unsigned char)status );
}

static void RaiseDtr( void )
{
   int status;

   status = inp( ComBase + MODEM_CNTRL );
   status |= 0x01;                  /* turn on DTR bit */
   outp( ComBase + MODEM_CNTRL, (unsigned char)status );
}

int ComRecChar( void )
{
   return QueueGet( Serial_In_Buffer );
}

int ComSendString( char *string )
{
   int retval;
   char *pointer;
   
   pointer = string;
   while (*pointer)
   {
      retval = QueuePut( Serial_Out_Buffer, *pointer );
      pointer++;
   }
   if (0x0 == (comm_status.modem & 0x40))
      RaiseDtr();
   outp( ComBase + INT_EN, RX_INT | TBE_INT | ERR_INT | RS_INT );
   return retval;
}

int ComSendChar( char character )
{
   int retval;

   /* interrupt driven send */
   if ( 0x0 == (comm_status.modem & 0x40))
      RaiseDtr();
   retval = QueuePut( Serial_Out_Buffer, character );
   if (- 1 != retval)
      outp( ComBase + INT_EN, RX_INT | TBE_INT | ERR_INT | RS_INT );
   return retval;
}

int ComStatus( void )
{
   unsigned status;
   unsigned retval;

   retval = inp( ComBase + LINE_STATUS );
   retval = retval << 8;
   status = inp( ComBase + MODEM_STATUS );
   retval = retval | status;
   if ( QueueEmpty( Serial_In_Buffer ))
      retval &= 0xFEFF;
   else  
      retval |= 0x0100;
   return (int)retval;
}

static void interrupt far serial( void )     /* interrupt handler */
{
   int temp;

   disable();
   while (1)
   {
      comm_status.intrupt = inp( ComBase + INT_ID );
      comm_status.intrupt &= 0x0f;
      switch (comm_status.intrupt)
      {
         case 0x00:                 /* modem interrupt */
            comm_status.modem = inp( ComBase + MODEM_STATUS );
            break;
         case 0x02:                 /* xmit interrupt */
            if (QueueEmpty( Serial_Out_Buffer ))
               outp( ComBase + INT_EN, RX_INT|ERR_INT|RS_INT );
            else
            {
               temp = QueueGet( Serial_Out_Buffer );
               if (- 1 != temp)
                  outp( ComBase + XMIT, temp );
            }
            break;
         case 0x04:                 /* receive interrupt */
            QueuePut(Serial_In_Buffer, (char)inp(ComBase + REC));
            break;
         case 0x06:                 /* line interrupt */
            comm_status.line = inp( ComBase + LINE_STATUS );
            inp( ComBase + REC );
            QueuePut( Serial_In_Buffer, '!' );
            break;
         default:                   /* No Mo` Left */
            comm_status.modem = inp( ComBase + MODEM_STATUS );
            outp( 0x20, 0x20 );
            enable();
            return;
      }   /* switch */
   }   /* while */
}
